@@ -27,6 +27,7 @@ package elasticsearch
 import (
 	"bytes"
 	"context"
+	"encoding/base64"
 	"encoding/json"
 	"errors"
 	"fmt"
@@ -45,7 +46,6 @@ import (
 
 	"go.temporal.io/server/common"
 	"go.temporal.io/server/common/config"
-	"go.temporal.io/server/common/convert"
 	"go.temporal.io/server/common/log"
 	"go.temporal.io/server/common/log/tag"
 	"go.temporal.io/server/common/metrics"
@@ -80,27 +80,13 @@ type (
 		// for ES scroll API
 		ScrollID string
 	}
-
-	visibilityRecord struct {
-		WorkflowID      string
-		RunID           string
-		WorkflowType    string
-		StartTime       int64
-		ExecutionTime   int64
-		CloseTime       int64
-		ExecutionStatus enumspb.WorkflowExecutionStatus
-		HistoryLength   int64
-		Memo            []byte
-		Encoding        string
-		TaskQueue       string
-		Attr            map[string]interface{}
-	}
 )
 
 var _ persistence.VisibilityStore = (*visibilityStore)(nil)
 
 var (
-	oneMilliSecondInNano = int64(1000)
+	ErrInvalidDuration         = errors.New("invalid duration format")
+	errUnexpectedJSONFieldType = errors.New("unexpected JSON field type")
 )
 
 // NewVisibilityStore create a visibility store connecting to ElasticSearch
@@ -147,8 +133,10 @@ func (s *visibilityStore) RecordWorkflowExecutionClosed(request *persistence.Int
 	visibilityTaskKey := getVisibilityTaskKey(request.ShardID, request.TaskID)
 	doc := s.generateESDoc(request.InternalVisibilityRequestBase, visibilityTaskKey)
 
-	doc[searchattribute.CloseTime] = request.CloseTimestamp
+	doc[searchattribute.CloseTime] = request.CloseTime
+	doc[searchattribute.ExecutionDuration] = request.CloseTime.Sub(request.ExecutionTime).Nanoseconds()
 	doc[searchattribute.HistoryLength] = request.HistoryLength
+	doc[searchattribute.StateTransitionCount] = request.StateTransitionCount
 
 	return s.addBulkIndexRequestAndWait(request.InternalVisibilityRequestBase, doc, visibilityTaskKey)
 }
@@ -200,8 +188,7 @@ func (s *visibilityStore) addBulkIndexRequestAndWait(
 func (s *visibilityStore) addBulkRequestAndWait(bulkRequest *client.BulkableRequest, visibilityTaskKey string) error {
 	s.checkProcessor()
 
-	ackCh := make(chan bool, 1)
-	s.processor.Add(bulkRequest, visibilityTaskKey, ackCh)
+	ackCh := s.processor.Add(bulkRequest, visibilityTaskKey)
 	ackTimeoutTimer := time.NewTimer(s.config.ESProcessorAckTimeout())
 	defer ackTimeoutTimer.Stop()
 
@@ -216,40 +203,6 @@ func (s *visibilityStore) addBulkRequestAndWait(bulkRequest *client.BulkableRequ
 	}
 }
 
-func (s *visibilityStore) generateESDoc(request *persistence.InternalVisibilityRequestBase, visibilityTaskKey string) map[string]interface{} {
-	doc := map[string]interface{}{
-		searchattribute.VisibilityTaskKey: visibilityTaskKey,
-		searchattribute.NamespaceID:       request.NamespaceID,
-		searchattribute.WorkflowID:        request.WorkflowID,
-		searchattribute.RunID:             request.RunID,
-		searchattribute.WorkflowType:      request.WorkflowTypeName,
-		searchattribute.StartTime:         request.StartTimestamp,
-		searchattribute.ExecutionTime:     request.ExecutionTimestamp,
-		searchattribute.ExecutionStatus:   request.Status,
-		searchattribute.TaskQueue:         request.TaskQueue,
-	}
-
-	if len(request.Memo.GetData()) > 0 {
-		doc[searchattribute.Memo] = request.Memo.GetData()
-		doc[searchattribute.Encoding] = request.Memo.GetEncodingType().String()
-	}
-
-	typeMap, err := s.searchAttributesProvider.GetSearchAttributes(s.index, false)
-	if err != nil {
-		s.logger.Error("Unable to read search attribute types.", tag.Error(err))
-		s.metricsClient.IncCounter(metrics.ElasticSearchVisibility, metrics.ESInvalidSearchAttribute)
-	}
-
-	attr, err := searchattribute.Decode(request.SearchAttributes, &typeMap)
-	if err != nil {
-		s.logger.Error("Unable to decode search attributes.", tag.Error(err))
-		s.metricsClient.IncCounter(metrics.ElasticSearchVisibility, metrics.ESInvalidSearchAttribute)
-	}
-	doc[searchattribute.Attr] = attr
-
-	return doc
-}
-
 func (s *visibilityStore) ListOpenWorkflowExecutions(
 	request *persistence.ListWorkflowExecutionsRequest) (*persistence.InternalListWorkflowExecutionsResponse, error) {
 	token, err := s.getNextPageToken(request.NextPageToken)
@@ -257,15 +210,14 @@ func (s *visibilityStore) ListOpenWorkflowExecutions(
 		return nil, err
 	}
 
-	query := elastic.NewBoolQuery().Must(elastic.NewMatchQuery(searchattribute.ExecutionStatus, int(enumspb.WORKFLOW_EXECUTION_STATUS_RUNNING)))
+	query := elastic.NewBoolQuery().Must(elastic.NewMatchQuery(searchattribute.ExecutionStatus, enumspb.WORKFLOW_EXECUTION_STATUS_RUNNING.String()))
 	searchResult, err := s.getSearchResult(request, token, query, true)
 	if err != nil {
 		return nil, serviceerror.NewInternal(fmt.Sprintf("ListOpenWorkflowExecutions failed. Error: %v", err))
 	}
 
 	isRecordValid := func(rec *persistence.VisibilityWorkflowExecutionInfo) bool {
-		startTime := rec.StartTime.UnixNano()
-		return request.EarliestStartTime <= startTime && startTime <= request.LatestStartTime
+		return !rec.StartTime.Before(request.EarliestStartTime) && !rec.StartTime.After(request.LatestStartTime)
 	}
 
 	return s.getListWorkflowExecutionsResponse(searchResult.Hits, token, request.PageSize, isRecordValid)
@@ -279,15 +231,14 @@ func (s *visibilityStore) ListClosedWorkflowExecutions(
 		return nil, err
 	}
 
-	executionStatusQuery := elastic.NewBoolQuery().MustNot(elastic.NewMatchQuery(searchattribute.ExecutionStatus, int(enumspb.WORKFLOW_EXECUTION_STATUS_RUNNING)))
+	executionStatusQuery := elastic.NewBoolQuery().MustNot(elastic.NewMatchQuery(searchattribute.ExecutionStatus, enumspb.WORKFLOW_EXECUTION_STATUS_RUNNING.String()))
 	searchResult, err := s.getSearchResult(request, token, executionStatusQuery, false)
 	if err != nil {
 		return nil, serviceerror.NewInternal(fmt.Sprintf("ListClosedWorkflowExecutions failed. Error: %v", err))
 	}
 
 	isRecordValid := func(rec *persistence.VisibilityWorkflowExecutionInfo) bool {
-		closeTime := rec.CloseTime.UnixNano()
-		return request.EarliestStartTime <= closeTime && closeTime <= request.LatestStartTime
+		return !rec.CloseTime.Before(request.EarliestStartTime) && !rec.CloseTime.After(request.LatestStartTime)
 	}
 
 	return s.getListWorkflowExecutionsResponse(searchResult.Hits, token, request.PageSize, isRecordValid)
@@ -302,15 +253,14 @@ func (s *visibilityStore) ListOpenWorkflowExecutionsByType(
 	}
 
 	query := elastic.NewBoolQuery().Must(elastic.NewMatchQuery(searchattribute.WorkflowType, request.WorkflowTypeName)).
-		Must(elastic.NewMatchQuery(searchattribute.ExecutionStatus, int(enumspb.WORKFLOW_EXECUTION_STATUS_RUNNING)))
+		Must(elastic.NewMatchQuery(searchattribute.ExecutionStatus, enumspb.WORKFLOW_EXECUTION_STATUS_RUNNING.String()))
 	searchResult, err := s.getSearchResult(&request.ListWorkflowExecutionsRequest, token, query, true)
 	if err != nil {
 		return nil, serviceerror.NewInternal(fmt.Sprintf("ListOpenWorkflowExecutionsByType failed. Error: %v", err))
 	}
 
 	isRecordValid := func(rec *persistence.VisibilityWorkflowExecutionInfo) bool {
-		startTime := rec.StartTime.UnixNano()
-		return request.EarliestStartTime <= startTime && startTime <= request.LatestStartTime
+		return !rec.StartTime.Before(request.EarliestStartTime) && !rec.StartTime.After(request.LatestStartTime)
 	}
 
 	return s.getListWorkflowExecutionsResponse(searchResult.Hits, token, request.PageSize, isRecordValid)
@@ -325,15 +275,14 @@ func (s *visibilityStore) ListClosedWorkflowExecutionsByType(
 	}
 
 	query := elastic.NewBoolQuery().Must(elastic.NewMatchQuery(searchattribute.WorkflowType, request.WorkflowTypeName)).
-		MustNot(elastic.NewMatchQuery(searchattribute.ExecutionStatus, int(enumspb.WORKFLOW_EXECUTION_STATUS_RUNNING)))
+		MustNot(elastic.NewMatchQuery(searchattribute.ExecutionStatus, enumspb.WORKFLOW_EXECUTION_STATUS_RUNNING.String()))
 	searchResult, err := s.getSearchResult(&request.ListWorkflowExecutionsRequest, token, query, false)
 	if err != nil {
 		return nil, serviceerror.NewInternal(fmt.Sprintf("ListClosedWorkflowExecutionsByType failed. Error: %v", err))
 	}
 
 	isRecordValid := func(rec *persistence.VisibilityWorkflowExecutionInfo) bool {
-		closeTime := rec.CloseTime.UnixNano()
-		return request.EarliestStartTime <= closeTime && closeTime <= request.LatestStartTime
+		return !rec.CloseTime.Before(request.EarliestStartTime) && !rec.CloseTime.After(request.LatestStartTime)
 	}
 
 	return s.getListWorkflowExecutionsResponse(searchResult.Hits, token, request.PageSize, isRecordValid)
@@ -348,15 +297,14 @@ func (s *visibilityStore) ListOpenWorkflowExecutionsByWorkflowID(
 	}
 
 	query := elastic.NewBoolQuery().Must(elastic.NewMatchQuery(searchattribute.WorkflowID, request.WorkflowID)).
-		Must(elastic.NewMatchQuery(searchattribute.ExecutionStatus, int(enumspb.WORKFLOW_EXECUTION_STATUS_RUNNING)))
+		Must(elastic.NewMatchQuery(searchattribute.ExecutionStatus, enumspb.WORKFLOW_EXECUTION_STATUS_RUNNING.String()))
 	searchResult, err := s.getSearchResult(&request.ListWorkflowExecutionsRequest, token, query, true)
 	if err != nil {
 		return nil, serviceerror.NewInternal(fmt.Sprintf("ListOpenWorkflowExecutionsByWorkflowID failed. Error: %v", err))
 	}
 
 	isRecordValid := func(rec *persistence.VisibilityWorkflowExecutionInfo) bool {
-		startTime := rec.StartTime.UnixNano()
-		return request.EarliestStartTime <= startTime && startTime <= request.LatestStartTime
+		return !rec.StartTime.Before(request.EarliestStartTime) && !rec.StartTime.After(request.LatestStartTime)
 	}
 
 	return s.getListWorkflowExecutionsResponse(searchResult.Hits, token, request.PageSize, isRecordValid)
@@ -371,15 +319,14 @@ func (s *visibilityStore) ListClosedWorkflowExecutionsByWorkflowID(
 	}
 
 	query := elastic.NewBoolQuery().Must(elastic.NewMatchQuery(searchattribute.WorkflowID, request.WorkflowID)).
-		MustNot(elastic.NewMatchQuery(searchattribute.ExecutionStatus, int(enumspb.WORKFLOW_EXECUTION_STATUS_RUNNING)))
+		MustNot(elastic.NewMatchQuery(searchattribute.ExecutionStatus, enumspb.WORKFLOW_EXECUTION_STATUS_RUNNING.String()))
 	searchResult, err := s.getSearchResult(&request.ListWorkflowExecutionsRequest, token, query, false)
 	if err != nil {
 		return nil, serviceerror.NewInternal(fmt.Sprintf("ListClosedWorkflowExecutionsByWorkflowID failed. Error: %v", err))
 	}
 
 	isRecordValid := func(rec *persistence.VisibilityWorkflowExecutionInfo) bool {
-		closeTime := rec.CloseTime.UnixNano()
-		return request.EarliestStartTime <= closeTime && closeTime <= request.LatestStartTime
+		return !rec.CloseTime.Before(request.EarliestStartTime) && !rec.CloseTime.After(request.LatestStartTime)
 	}
 
 	return s.getListWorkflowExecutionsResponse(searchResult.Hits, token, request.PageSize, isRecordValid)
@@ -393,15 +340,14 @@ func (s *visibilityStore) ListClosedWorkflowExecutionsByStatus(
 		return nil, err
 	}
 
-	query := elastic.NewBoolQuery().Must(elastic.NewMatchQuery(searchattribute.ExecutionStatus, int32(request.Status)))
+	query := elastic.NewBoolQuery().Must(elastic.NewMatchQuery(searchattribute.ExecutionStatus, request.Status.String()))
 	searchResult, err := s.getSearchResult(&request.ListWorkflowExecutionsRequest, token, query, false)
 	if err != nil {
 		return nil, serviceerror.NewInternal(fmt.Sprintf("ListClosedWorkflowExecutionsByStatus failed. Error: %v", err))
 	}
 
 	isRecordValid := func(rec *persistence.VisibilityWorkflowExecutionInfo) bool {
-		closeTime := rec.CloseTime.UnixNano()
-		return request.EarliestStartTime <= closeTime && closeTime <= request.LatestStartTime
+		return !rec.CloseTime.Before(request.EarliestStartTime) && !rec.CloseTime.After(request.LatestStartTime)
 	}
 
 	return s.getListWorkflowExecutionsResponse(searchResult.Hits, token, request.PageSize, isRecordValid)
@@ -411,7 +357,7 @@ func (s *visibilityStore) GetClosedWorkflowExecution(
 	request *persistence.GetClosedWorkflowExecutionRequest) (*persistence.InternalGetClosedWorkflowExecutionResponse, error) {
 
 	matchNamespaceQuery := elastic.NewMatchQuery(searchattribute.NamespaceID, request.NamespaceID)
-	executionStatusQuery := elastic.NewMatchQuery(searchattribute.ExecutionStatus, int(enumspb.WORKFLOW_EXECUTION_STATUS_RUNNING))
+	executionStatusQuery := elastic.NewMatchQuery(searchattribute.ExecutionStatus, enumspb.WORKFLOW_EXECUTION_STATUS_RUNNING.String())
 	matchWorkflowIDQuery := elastic.NewMatchQuery(searchattribute.WorkflowID, request.Execution.GetWorkflowId())
 	boolQuery := elastic.NewBoolQuery().Must(matchNamespaceQuery).MustNot(executionStatusQuery).Must(matchWorkflowIDQuery)
 	rid := request.Execution.GetRunId()
@@ -431,12 +377,17 @@ func (s *visibilityStore) GetClosedWorkflowExecution(
 	}
 
 	response := &persistence.InternalGetClosedWorkflowExecutionResponse{}
-	actualHits := searchResult.Hits.Hits
-	if len(actualHits) == 0 {
+	if len(searchResult.Hits.Hits) == 0 {
 		return response, nil
 	}
-	response.Execution = s.convertSearchResultToVisibilityRecord(actualHits[0])
 
+	typeMap, err := s.searchAttributesProvider.GetSearchAttributes(s.index, false)
+	if err != nil {
+		s.logger.Error("Unable to read search attribute types.", tag.Error(err))
+		return nil, err
+	}
+
+	response.Execution = s.parseESDoc(searchResult.Hits.Hits[0], typeMap)
 	return response, nil
 }
 
@@ -517,32 +468,35 @@ func (s *visibilityStore) CountWorkflowExecutions(request *persistence.CountWork
 	return response, nil
 }
 
+// TODO (alex): move this to separate file
+// TODO (alex): consider replacing this code with Elasticsearch SQL from X-Pack: https://www.elastic.co/what-is/open-x-pack.
 const (
-	jsonMissingCloseTime     = `{"missing":{"field":"CloseTime"}}`
-	jsonRangeOnExecutionTime = `{"range":{"ExecutionTime":`
-	jsonSortForOpen          = `[{"StartTime":"desc"},{"RunId":"desc"}]`
-	jsonSortWithTieBreaker   = `{"RunId":"desc"}`
+	jsonMissingCloseTime   = `{"missing":{"field":"CloseTime"}}`
+	jsonSortForOpen        = `[{"StartTime":"desc"},{"RunId":"desc"}]`
+	jsonSortWithTieBreaker = `{"RunId":"desc"}`
 
 	dslFieldSort        = "sort"
 	dslFieldSearchAfter = "search_after"
 	dslFieldFrom        = "from"
 	dslFieldSize        = "size"
-
-	defaultDateTimeFormat = time.RFC3339 // used for converting UnixNano to string like 2018-02-15T16:16:36-08:00
 )
 
 var (
-	timeKeys = map[string]bool{
-		"StartTime":     true,
-		"CloseTime":     true,
-		"ExecutionTime": true,
+	timeKeys = map[string]struct{}{
+		searchattribute.StartTime:     {},
+		searchattribute.CloseTime:     {},
+		searchattribute.ExecutionTime: {},
 	}
-	rangeKeys = map[string]bool{
-		"from":  true,
-		"to":    true,
-		"gt":    true,
-		"lt":    true,
-		"query": true,
+	rangeKeys = map[string]struct{}{
+		"from":  {},
+		"to":    {},
+		"gt":    {},
+		"lt":    {},
+		"query": {},
+	}
+
+	exactMatchKeys = map[string]struct{}{
+		"query": {},
 	}
 )
 
@@ -595,9 +549,7 @@ func (s *visibilityStore) getESQueryDSL(request *persistence.ListWorkflowExecuti
 		dsl.Set(dslFieldFrom, fastjson.MustParse(strconv.Itoa(token.From)))
 	}
 
-	dslStr := cleanDSL(dsl.String())
-
-	return dslStr, nil
+	return dsl.String(), nil
 }
 
 func getSQLFromListRequest(request *persistence.ListWorkflowExecutionsRequestV2) string {
@@ -623,6 +575,10 @@ func getSQLFromCountRequest(request *persistence.CountWorkflowExecutionsRequest)
 	return sql
 }
 
+// getCustomizedDSLFromSQL converts SQL-ish query to Elasticsearch JSON query.
+// This is primarily done by `elasticsql` package.
+// Queries like `ExecutionStatus="Running"` are converted to: `{"query":{"bool":{"must":[{"match_phrase":{"ExecutionStatus":{"query":"Running"}}}]}},"from":0,"size":20}`.
+// Then `fastjson` parse this JSON and substitute some values.
 func getCustomizedDSLFromSQL(sql string, namespaceID string) (*fastjson.Value, error) {
 	dslStr, _, err := elasticsql.Convert(sql)
 	if err != nil {
@@ -636,13 +592,16 @@ func getCustomizedDSLFromSQL(sql string, namespaceID string) (*fastjson.Value, e
 	if strings.Contains(dslStr, jsonMissingCloseTime) { // isOpen
 		dsl = replaceQueryForOpen(dsl)
 	}
-	if strings.Contains(dslStr, jsonRangeOnExecutionTime) {
-		addQueryForExecutionTime(dsl)
-	}
 	addNamespaceToQuery(dsl, namespaceID)
 	if err := processAllValuesForKey(dsl, timeKeyFilter, timeProcessFunc); err != nil {
 		return nil, err
 	}
+	if err := processAllValuesForKey(dsl, statusKeyFilter, statusProcessFunc); err != nil {
+		return nil, err
+	}
+	if err := processAllValuesForKey(dsl, durationKeyFilter, durationProcessFunc); err != nil {
+		return nil, err
+	}
 	return dsl, nil
 }
 
@@ -656,11 +615,6 @@ func replaceQueryForOpen(dsl *fastjson.Value) *fastjson.Value {
 	return dsl
 }
 
-func addQueryForExecutionTime(dsl *fastjson.Value) {
-	executionTimeQueryString := `{"range" : {"ExecutionTime" : {"gt" : "0"}}}`
-	addMustQuery(dsl, executionTimeQueryString)
-}
-
 func addNamespaceToQuery(dsl *fastjson.Value, namespaceID string) {
 	if len(namespaceID) == 0 {
 		return
@@ -707,9 +661,6 @@ func (s *visibilityStore) processSortField(dsl *fastjson.Value) (string, error)
 }
 
 func (s *visibilityStore) getFieldType(fieldName string) enumspb.IndexedValueType {
-	if strings.HasPrefix(fieldName, searchattribute.Attr) {
-		fieldName = fieldName[len(searchattribute.Attr)+1:] // remove prefix
-	}
 	searchAttributes, err := s.searchAttributesProvider.GetSearchAttributes(s.index, false)
 	if err != nil {
 		s.logger.Error("Unable to read search attribute types.", tag.Error(err))
@@ -790,34 +741,31 @@ func (s *visibilityStore) getNextPageToken(token []byte) (*visibilityPageToken,
 func (s *visibilityStore) getSearchResult(request *persistence.ListWorkflowExecutionsRequest, token *visibilityPageToken,
 	boolQuery *elastic.BoolQuery, overStartTime bool) (*elastic.SearchResult, error) {
 
-	matchNamespaceQuery := elastic.NewMatchQuery(searchattribute.NamespaceID, request.NamespaceID)
-	var rangeQuery *elastic.RangeQuery
-	if overStartTime {
-		rangeQuery = elastic.NewRangeQuery(searchattribute.StartTime)
-	} else {
-		rangeQuery = elastic.NewRangeQuery(searchattribute.CloseTime)
-	}
-	// ElasticSearch v6 is unable to precisely compare time, have to manually add resolution 1ms to time range.
-	// Also has to use string instead of int64 to avoid data conversion issue,
-	// 9223372036854775807 to 9223372036854776000 (long overflow)
-	if request.LatestStartTime > math.MaxInt64-oneMilliSecondInNano { // prevent latestTime overflow
-		request.LatestStartTime = math.MaxInt64 - oneMilliSecondInNano
-	}
-	if request.EarliestStartTime < math.MinInt64+oneMilliSecondInNano { // prevent earliestTime overflow
-		request.EarliestStartTime = math.MinInt64 + oneMilliSecondInNano
-	}
-	earliestTimeStr := convert.Int64ToString(request.EarliestStartTime - oneMilliSecondInNano)
-	latestTimeStr := convert.Int64ToString(request.LatestStartTime + oneMilliSecondInNano)
-	rangeQuery = rangeQuery.
-		Gte(earliestTimeStr).
-		Lte(latestTimeStr)
-
 	query := elastic.NewBoolQuery()
 	if boolQuery != nil {
 		*query = *boolQuery
 	}
 
-	query = query.Must(matchNamespaceQuery).Filter(rangeQuery)
+	matchNamespaceQuery := elastic.NewMatchQuery(searchattribute.NamespaceID, request.NamespaceID)
+	query = query.Must(matchNamespaceQuery)
+
+	if !request.EarliestStartTime.IsZero() || !request.LatestStartTime.IsZero() {
+		var rangeQuery *elastic.RangeQuery
+		if overStartTime {
+			rangeQuery = elastic.NewRangeQuery(searchattribute.StartTime)
+		} else {
+			rangeQuery = elastic.NewRangeQuery(searchattribute.CloseTime)
+		}
+
+		if !request.EarliestStartTime.IsZero() {
+			rangeQuery = rangeQuery.Gte(request.EarliestStartTime)
+		}
+
+		if !request.LatestStartTime.IsZero() {
+			rangeQuery = rangeQuery.Lte(request.LatestStartTime)
+		}
+		query = query.Filter(rangeQuery)
+	}
 
 	ctx := context.Background()
 	params := &client.SearchParameters{
@@ -844,17 +792,19 @@ func (s *visibilityStore) getScanWorkflowExecutionsResponse(searchHits *elastic.
 	token *visibilityPageToken, pageSize int, scrollID string, isLastPage bool) (
 	*persistence.InternalListWorkflowExecutionsResponse, error) {
 
-	response := &persistence.InternalListWorkflowExecutionsResponse{}
-	actualHits := searchHits.Hits
-	numOfActualHits := len(actualHits)
-
-	response.Executions = make([]*persistence.VisibilityWorkflowExecutionInfo, 0)
-	for i := 0; i < numOfActualHits; i++ {
-		workflowExecutionInfo := s.convertSearchResultToVisibilityRecord(actualHits[i])
-		response.Executions = append(response.Executions, workflowExecutionInfo)
+	typeMap, err := s.searchAttributesProvider.GetSearchAttributes(s.index, false)
+	if err != nil {
+		s.logger.Error("Unable to read search attribute types.", tag.Error(err))
+		return nil, err
 	}
 
-	if numOfActualHits == pageSize && !isLastPage {
+	response := &persistence.InternalListWorkflowExecutionsResponse{}
+	response.Executions = make([]*persistence.VisibilityWorkflowExecutionInfo, len(searchHits.Hits))
+	for i := 0; i < len(searchHits.Hits); i++ {
+		response.Executions[i] = s.parseESDoc(searchHits.Hits[i], typeMap)
+	}
+
+	if len(searchHits.Hits) == pageSize && !isLastPage {
 		nextPageToken, err := s.serializePageToken(&visibilityPageToken{ScrollID: scrollID})
 		if err != nil {
 			return nil, err
@@ -869,31 +819,37 @@ func (s *visibilityStore) getScanWorkflowExecutionsResponse(searchHits *elastic.
 func (s *visibilityStore) getListWorkflowExecutionsResponse(searchHits *elastic.SearchHits,
 	token *visibilityPageToken, pageSize int, isRecordValid func(rec *persistence.VisibilityWorkflowExecutionInfo) bool) (*persistence.InternalListWorkflowExecutionsResponse, error) {
 
-	response := &persistence.InternalListWorkflowExecutionsResponse{}
-	actualHits := searchHits.Hits
-	numOfActualHits := len(actualHits)
+	typeMap, err := s.searchAttributesProvider.GetSearchAttributes(s.index, false)
+	if err != nil {
+		s.logger.Error("Unable to read search attribute types.", tag.Error(err))
+		return nil, err
+	}
 
-	response.Executions = make([]*persistence.VisibilityWorkflowExecutionInfo, 0)
-	for i := 0; i < numOfActualHits; i++ {
-		workflowExecutionInfo := s.convertSearchResultToVisibilityRecord(actualHits[i])
+	response := &persistence.InternalListWorkflowExecutionsResponse{}
+
+	response.Executions = make([]*persistence.VisibilityWorkflowExecutionInfo, 0, len(searchHits.Hits))
+	for _, hit := range searchHits.Hits {
+		workflowExecutionInfo := s.parseESDoc(hit, typeMap)
+		// ES6 uses "date" data type not "date_nanos". It truncates dates using milliseconds and might return extra rows.
+		// For example: 2021-06-12T00:21:43.159739259Z fits 2021-06-12T00:21:43.158Z...2021-06-12T00:21:43.159Z range lte/gte query.
+		// Therefore these records needs to be filtered out on the client side to support nanos precision.
+		// After ES6 deprecation isRecordValid can be removed.
 		if isRecordValid == nil || isRecordValid(workflowExecutionInfo) {
-			// for old APIs like ListOpenWorkflowExecutions, we added 1 ms to range query to overcome ES limitation
-			// (see getSearchResult function), but manually dropped records beyond request range here.
 			response.Executions = append(response.Executions, workflowExecutionInfo)
 		}
 	}
 
-	if numOfActualHits == pageSize { // this means the response is not the last page
+	if len(searchHits.Hits) == pageSize { // this means the response is not the last page
 		var nextPageToken []byte
 		var err error
 
 		// ES Search API support pagination using From and PageSize, but has limit that From+PageSize cannot exceed a threshold
 		// to retrieve deeper pages, use ES SearchAfter
 		if searchHits.TotalHits.Value <= int64(s.config.ESIndexMaxResultWindow()-pageSize) { // use ES Search From+Size
-			nextPageToken, err = s.serializePageToken(&visibilityPageToken{From: token.From + numOfActualHits})
+			nextPageToken, err = s.serializePageToken(&visibilityPageToken{From: token.From + len(searchHits.Hits)})
 		} else { // use ES Search After
 			var sortVal interface{}
-			sortVals := actualHits[len(response.Executions)-1].Sort
+			sortVals := searchHits.Hits[len(response.Executions)-1].Sort
 			sortVal = sortVals[0]
 			tieBreaker := sortVals[1].(string)
 
@@ -929,42 +885,196 @@ func (s *visibilityStore) serializePageToken(token *visibilityPageToken) ([]byte
 	return data, nil
 }
 
-func (s *visibilityStore) convertSearchResultToVisibilityRecord(hit *elastic.SearchHit) *persistence.VisibilityWorkflowExecutionInfo {
-	var source *visibilityRecord
-	// TODO (alex): use hit.Fields instead
-	err := json.Unmarshal(hit.Source, &source)
-	if err != nil { // log and skip error
-		s.logger.Error("unable to unmarshal search hit source",
-			tag.Error(err), tag.ESDocID(hit.Id))
+func (s *visibilityStore) generateESDoc(request *persistence.InternalVisibilityRequestBase, visibilityTaskKey string) map[string]interface{} {
+	doc := map[string]interface{}{
+		searchattribute.VisibilityTaskKey: visibilityTaskKey,
+		searchattribute.NamespaceID:       request.NamespaceID,
+		searchattribute.WorkflowID:        request.WorkflowID,
+		searchattribute.RunID:             request.RunID,
+		searchattribute.WorkflowType:      request.WorkflowTypeName,
+		searchattribute.StartTime:         request.StartTime,
+		searchattribute.ExecutionTime:     request.ExecutionTime,
+		searchattribute.ExecutionStatus:   request.Status.String(),
+		searchattribute.TaskQueue:         request.TaskQueue,
+	}
+
+	if len(request.Memo.GetData()) > 0 {
+		doc[searchattribute.Memo] = request.Memo.GetData()
+		doc[searchattribute.MemoEncoding] = request.Memo.GetEncodingType().String()
+	}
+
+	typeMap, err := s.searchAttributesProvider.GetSearchAttributes(s.index, false)
+	if err != nil {
+		s.logger.Error("Unable to read search attribute types.", tag.Error(err))
+	}
+
+	searchAttributes, err := searchattribute.Decode(request.SearchAttributes, &typeMap)
+	if err != nil {
+		s.logger.Error("Unable to decode search attributes.", tag.Error(err))
+	}
+	for saName, saValue := range searchAttributes {
+		doc[saName] = saValue
+	}
+
+	return doc
+}
+
+func (s *visibilityStore) parseESDoc(hit *elastic.SearchHit, saTypeMap searchattribute.NameTypeMap) *persistence.VisibilityWorkflowExecutionInfo {
+	logParseError := func(fieldName string, fieldValue interface{}, err error, docID string) {
+		s.logger.Error("Unable to parse Elasticsearch document field.", tag.Name(fieldName), tag.Value(fieldValue), tag.Error(err), tag.ESDocID(docID))
+		s.metricsClient.IncCounter(metrics.ElasticsearchVisibility, metrics.ElasticsearchInvalidSearchAttributeCount)
+	}
+
+	var sourceMap map[string]interface{}
+	d := json.NewDecoder(bytes.NewReader(hit.Source))
+	// Very important line. See finishParseJSONValue bellow.
+	d.UseNumber()
+	if err := d.Decode(&sourceMap); err != nil {
+		s.logger.Error("Unable to JSON unmarshal Elasticsearch SearchHit.Source.", tag.Error(err), tag.ESDocID(hit.Id))
 		return nil
 	}
 
-	record := &persistence.VisibilityWorkflowExecutionInfo{
-		WorkflowID:       source.WorkflowID,
-		RunID:            source.RunID,
-		TypeName:         source.WorkflowType,
-		StartTime:        time.Unix(0, source.StartTime).UTC(),
-		ExecutionTime:    time.Unix(0, source.ExecutionTime).UTC(),
-		Memo:             persistence.NewDataBlob(source.Memo, source.Encoding),
-		TaskQueue:        source.TaskQueue,
-		SearchAttributes: source.Attr,
-		Status:           source.ExecutionStatus,
+	var isValidType bool
+	var memo []byte
+	var memoEncoding string
+	record := &persistence.VisibilityWorkflowExecutionInfo{}
+	for fieldName, fieldValue := range sourceMap {
+		switch fieldName {
+		case searchattribute.NamespaceID,
+			searchattribute.ExecutionDuration,
+			searchattribute.VisibilityTaskKey:
+			// Ignore these fields.
+			continue
+		case searchattribute.Memo:
+			var memoStr string
+			if memoStr, isValidType = fieldValue.(string); !isValidType {
+				logParseError(fieldName, fieldValue, fmt.Errorf("%w: expected string got %T", errUnexpectedJSONFieldType, fieldValue), hit.Id)
+			}
+			var err error
+			if memo, err = base64.StdEncoding.DecodeString(memoStr); err != nil {
+				logParseError(fieldName, memoStr[:10], err, hit.Id)
+			}
+			continue
+		case searchattribute.MemoEncoding:
+			if memoEncoding, isValidType = fieldValue.(string); !isValidType {
+				logParseError(fieldName, fieldValue, fmt.Errorf("%w: expected string got %T", errUnexpectedJSONFieldType, fieldValue), hit.Id)
+			}
+			continue
+		}
+
+		fieldType, err := saTypeMap.GetType(fieldName)
+		if err != nil {
+			// Silently ignore ErrInvalidName because it indicates unknown field in Elasticsearch document.
+			if !errors.Is(err, searchattribute.ErrInvalidName) {
+				s.logger.Error("Unable to get type for Elasticsearch document field.", tag.Name(fieldName), tag.Error(err), tag.ESDocID(hit.Id))
+			}
+			continue
+		}
+
+		fieldValueParsed, err := finishParseJSONValue(fieldValue, fieldType)
+		if err != nil {
+			logParseError(fieldName, fieldValue, err, hit.Id)
+			continue
+		}
+
+		switch fieldName {
+		case searchattribute.WorkflowID:
+			record.WorkflowID = fieldValueParsed.(string)
+		case searchattribute.RunID:
+			record.RunID = fieldValueParsed.(string)
+		case searchattribute.WorkflowType:
+			record.TypeName = fieldValue.(string)
+		case searchattribute.StartTime:
+			record.StartTime = fieldValueParsed.(time.Time)
+		case searchattribute.ExecutionTime:
+			record.ExecutionTime = fieldValueParsed.(time.Time)
+		case searchattribute.CloseTime:
+			record.CloseTime = fieldValueParsed.(time.Time)
+		case searchattribute.TaskQueue:
+			record.TaskQueue = fieldValueParsed.(string)
+		case searchattribute.ExecutionStatus:
+			record.Status = enumspb.WorkflowExecutionStatus(enumspb.WorkflowExecutionStatus_value[fieldValueParsed.(string)])
+		case searchattribute.HistoryLength:
+			record.HistoryLength = fieldValueParsed.(int64)
+		case searchattribute.StateTransitionCount:
+			record.StateTransitionCount = fieldValueParsed.(int64)
+		default:
+			// All custom search attributes are handled here.
+			if record.SearchAttributes == nil {
+				record.SearchAttributes = map[string]interface{}{}
+			}
+			record.SearchAttributes[fieldName] = fieldValueParsed
+		}
 	}
-	if source.CloseTime != 0 {
-		record.CloseTime = time.Unix(0, source.CloseTime).UTC()
-		record.HistoryLength = source.HistoryLength
+
+	if memoEncoding != "" {
+		record.Memo = persistence.NewDataBlob(memo, memoEncoding)
+	} else if memo != nil {
+		s.logger.Error("Field is missing in Elasticsearch document.", tag.Name(searchattribute.MemoEncoding), tag.ESDocID(hit.Id))
+		s.metricsClient.IncCounter(metrics.ElasticsearchVisibility, metrics.ElasticsearchInvalidSearchAttributeCount)
 	}
 
 	return record
 }
 
+// finishParseJSONValue finishes JSON parsing after json.Decode.
+// json.Decode returns:
+//     bool, for JSON booleans
+//     json.Number, for JSON numbers (because of d.UseNumber())
+//     string, for JSON strings
+//     []interface{}, for JSON arrays
+//     map[string]interface{}, for JSON objects (should never be a case)
+//     nil for JSON null
+func finishParseJSONValue(val interface{}, t enumspb.IndexedValueType) (interface{}, error) {
+	// Custom search attributes support array of particular type.
+	if arrayValue, isArray := val.([]interface{}); isArray {
+		retArray := make([]interface{}, len(arrayValue))
+		var lastErr error
+		for i := 0; i < len(retArray); i++ {
+			retArray[i], lastErr = finishParseJSONValue(arrayValue[i], t)
+		}
+		return retArray, lastErr
+	}
+
+	switch t {
+	case enumspb.INDEXED_VALUE_TYPE_STRING, enumspb.INDEXED_VALUE_TYPE_KEYWORD, enumspb.INDEXED_VALUE_TYPE_DATETIME:
+		stringVal, isString := val.(string)
+		if !isString {
+			return nil, fmt.Errorf("%w: expected string got %T", errUnexpectedJSONFieldType, val)
+		}
+		if t == enumspb.INDEXED_VALUE_TYPE_DATETIME {
+			return time.Parse(time.RFC3339Nano, stringVal)
+		}
+		return stringVal, nil
+	case enumspb.INDEXED_VALUE_TYPE_INT, enumspb.INDEXED_VALUE_TYPE_DOUBLE:
+		numberVal, isNumber := val.(json.Number)
+		if !isNumber {
+			return nil, fmt.Errorf("%w: expected json.Number got %T", errUnexpectedJSONFieldType, val)
+		}
+		if t == enumspb.INDEXED_VALUE_TYPE_INT {
+			return numberVal.Int64()
+		}
+		return numberVal.Float64()
+	case enumspb.INDEXED_VALUE_TYPE_BOOL:
+		boolVal, isBool := val.(bool)
+		if !isBool {
+			return nil, fmt.Errorf("%w: expected bool got %T", errUnexpectedJSONFieldType, val)
+		}
+		return boolVal, nil
+	}
+
+	panic(fmt.Sprintf("Unknown field type: %v", t))
+}
+
 func checkPageSize(request *persistence.ListWorkflowExecutionsRequestV2) {
 	if request.PageSize == 0 {
 		request.PageSize = 1000
 	}
 }
 
-func processAllValuesForKey(dsl *fastjson.Value, keyFilter func(k string) bool,
+func processAllValuesForKey(
+	dsl *fastjson.Value,
+	keyFilter func(k string) bool,
 	processFunc func(obj *fastjson.Object, key string, v *fastjson.Value) error,
 ) error {
 	switch dsl.Type() {
@@ -976,7 +1086,7 @@ func processAllValuesForKey(dsl *fastjson.Value, keyFilter func(k string) bool,
 		}
 	case fastjson.TypeObject:
 		objectVal := dsl.GetObject()
-		keys := []string{}
+		var keys []string
 		objectVal.Visit(func(key []byte, val *fastjson.Value) {
 			keys = append(keys, string(key))
 		})
@@ -1000,35 +1110,104 @@ func processAllValuesForKey(dsl *fastjson.Value, keyFilter func(k string) bool,
 }
 
 func timeKeyFilter(key string) bool {
-	return timeKeys[key]
+	_, ok := timeKeys[key]
+	return ok
 }
 
-func timeProcessFunc(obj *fastjson.Object, key string, value *fastjson.Value) error {
-	return processAllValuesForKey(value, func(key string) bool {
-		return rangeKeys[key]
-	}, func(obj *fastjson.Object, key string, v *fastjson.Value) error {
-		timeStr := string(v.GetStringBytes())
+func timeProcessFunc(_ *fastjson.Object, _ string, value *fastjson.Value) error {
+	return processAllValuesForKey(
+		value,
+		func(key string) bool {
+			_, ok := rangeKeys[key]
+			return ok
+		},
+		func(obj *fastjson.Object, key string, v *fastjson.Value) error {
+			timeStr := string(v.GetStringBytes())
 
-		// first check if already in int64 format
-		if _, err := strconv.ParseInt(timeStr, 10, 64); err == nil {
+			// To support dates passed as int64 "nanoseconds since epoch".
+			if nanos, err := strconv.ParseInt(timeStr, 10, 64); err == nil {
+				obj.Set(key, fastjson.MustParse(fmt.Sprintf(`"%s"`, time.Unix(0, nanos).UTC().Format(time.RFC3339Nano))))
+			}
 			return nil
-		}
-
-		// try to parse time
-		parsedTime, err := time.Parse(defaultDateTimeFormat, timeStr)
-		if err != nil {
-			return err
-		}
-
-		obj.Set(key, fastjson.MustParse(fmt.Sprintf(`"%v"`, parsedTime.UnixNano())))
-		return nil
-	})
+		})
 }
 
-// elasticsql may transfer `Attr.Name` to "`Attr.Name`" instead of "Attr.Name" in dsl in some operator like "between and"
-// this function is used to clean up
-func cleanDSL(input string) string {
-	var re = regexp.MustCompile("(`)(Attr.\\w+)(`)")
-	result := re.ReplaceAllString(input, `$2`)
-	return result
+// statusKeyFilter catch `ExecutionStatus` key and sends its value `{"query":"Running"}` to the statusProcessFunc.
+func statusKeyFilter(key string) bool {
+	return key == searchattribute.ExecutionStatus
+}
+
+// statusProcessFunc treats passed value as regular JSON and calls processAllValuesForKey for it.
+// keyFilterFunc func catches `query` key and call `processFunc` with value "Running".
+// In case of string it is just ignored but if it is a `int`, it gets converted to string and set back to `obj`.
+func statusProcessFunc(_ *fastjson.Object, _ string, value *fastjson.Value) error {
+	return processAllValuesForKey(
+		value,
+		func(key string) bool {
+			_, ok := exactMatchKeys[key]
+			return ok
+		},
+		func(obj *fastjson.Object, key string, v *fastjson.Value) error {
+			statusStr := string(v.GetStringBytes())
+
+			// To support statuses passed as integers for backward compatibility.
+			// Might be removed one day (added 6/15/21).
+			if statusInt, err := strconv.ParseInt(statusStr, 10, 32); err == nil {
+				statusStr = enumspb.WorkflowExecutionStatus_name[int32(statusInt)]
+				obj.Set(key, fastjson.MustParse(fmt.Sprintf(`"%s"`, statusStr)))
+			}
+			return nil
+		})
+}
+func durationKeyFilter(key string) bool {
+	return key == searchattribute.ExecutionDuration
+}
+
+func durationProcessFunc(_ *fastjson.Object, _ string, value *fastjson.Value) error {
+	return processAllValuesForKey(
+		value,
+		func(key string) bool {
+			_, ok := rangeKeys[key]
+			return ok
+		},
+		func(obj *fastjson.Object, key string, v *fastjson.Value) error {
+			durationStr := string(v.GetStringBytes())
+
+			// To support durations passed as golang durations such as "300ms", "-1.5h" or "2h45m".
+			// Valid time units are "ns", "us" (or "µs"), "ms", "s", "m", "h".
+			if duration, err := time.ParseDuration(durationStr); err == nil {
+				obj.Set(key, fastjson.MustParse(strconv.FormatInt(duration.Nanoseconds(), 10)))
+				return nil
+			}
+
+			// To support "hh:mm:ss" durations.
+			durationNanos, err := parseHHMMSSDuration(durationStr)
+			if errors.Is(err, ErrInvalidDuration) {
+				return err
+			}
+			if err == nil {
+				obj.Set(key, fastjson.MustParse(strconv.FormatInt(durationNanos, 10)))
+			}
+
+			return nil
+		})
+}
+
+func parseHHMMSSDuration(d string) (int64, error) {
+	var hours, minutes, seconds, nanos int64
+	_, err := fmt.Sscanf(d, "%d:%d:%d", &hours, &minutes, &seconds)
+	if err != nil {
+		return 0, err
+	}
+	if hours < 0 {
+		return 0, fmt.Errorf("%w: hours must be positive number", ErrInvalidDuration)
+	}
+	if minutes < 0 || minutes > 59 {
+		return 0, fmt.Errorf("%w: minutes must be from 0 to 59", ErrInvalidDuration)
+	}
+	if seconds < 0 || seconds > 59 {
+		return 0, fmt.Errorf("%w: seconds must be from 0 to 59", ErrInvalidDuration)
+	}
+
+	return hours*int64(time.Hour) + minutes*int64(time.Minute) + seconds*int64(time.Second) + nanos, nil
 }
