@@ -48,6 +48,7 @@ import (
 	"go.temporal.io/server/common/primitives/timestamp"
 	"go.temporal.io/server/common/searchattribute"
 	"go.temporal.io/server/service/history/configs"
+	"go.temporal.io/server/service/history/workflow"
 )
 
 type (
@@ -65,6 +66,9 @@ type (
 		blobSizeLimitWarn  int
 		blobSizeLimitError int
 
+		memoSizeLimitWarn  int
+		memoSizeLimitError int
+
 		historySizeLimitWarn  int
 		historySizeLimitError int
 
@@ -72,7 +76,7 @@ type (
 		historyCountLimitError int
 
 		completedID               int64
-		mutableState              mutableState
+		mutableState              workflow.MutableState
 		searchAttributesValidator *searchattribute.Validator
 		executionStats            *persistencespb.ExecutionStats
 		metricsScope              metrics.Scope
@@ -103,12 +107,14 @@ func newCommandAttrValidator(
 func newWorkflowSizeChecker(
 	blobSizeLimitWarn int,
 	blobSizeLimitError int,
+	memoSizeLimitWarn int,
+	memoSizeLimitError int,
 	historySizeLimitWarn int,
 	historySizeLimitError int,
 	historyCountLimitWarn int,
 	historyCountLimitError int,
 	completedID int64,
-	mutableState mutableState,
+	mutableState workflow.MutableState,
 	searchAttributesValidator *searchattribute.Validator,
 	executionStats *persistencespb.ExecutionStats,
 	metricsScope metrics.Scope,
@@ -117,6 +123,8 @@ func newWorkflowSizeChecker(
 	return &workflowSizeChecker{
 		blobSizeLimitWarn:         blobSizeLimitWarn,
 		blobSizeLimitError:        blobSizeLimitError,
+		memoSizeLimitWarn:         memoSizeLimitWarn,
+		memoSizeLimitError:        memoSizeLimitError,
 		historySizeLimitWarn:      historySizeLimitWarn,
 		historySizeLimitError:     historySizeLimitError,
 		historyCountLimitWarn:     historyCountLimitWarn,
@@ -164,6 +172,40 @@ func (c *workflowSizeChecker) failWorkflowIfPayloadSizeExceedsLimit(
 	return true, nil
 }
 
+func (c *workflowSizeChecker) failWorkflowIfMemoSizeExceedsLimit(
+	commandTypeTag metrics.Tag,
+	memoSize int,
+	message string,
+) (bool, error) {
+
+	executionInfo := c.mutableState.GetExecutionInfo()
+	executionState := c.mutableState.GetExecutionState()
+	err := common.CheckEventBlobSizeLimit(
+		memoSize,
+		c.memoSizeLimitWarn,
+		c.memoSizeLimitError,
+		executionInfo.NamespaceId,
+		executionInfo.WorkflowId,
+		executionState.RunId,
+		c.metricsScope.Tagged(commandTypeTag),
+		c.logger,
+		tag.BlobSizeViolationOperation(commandTypeTag.Value()),
+	)
+	if err == nil {
+		return false, nil
+	}
+
+	attributes := &commandpb.FailWorkflowExecutionCommandAttributes{
+		Failure: failure.NewServerFailure(message, true),
+	}
+
+	if _, err := c.mutableState.AddFailWorkflowEvent(c.completedID, enumspb.RETRY_STATE_NON_RETRYABLE_FAILURE, attributes); err != nil {
+		return false, err
+	}
+
+	return true, nil
+}
+
 func (c *workflowSizeChecker) failWorkflowIfSearchAttributesSizeExceedsLimit(
 	searchAttributes *commonpb.SearchAttributes,
 	namespace string,
