@@ -31,6 +31,7 @@ import (
 	"fmt"
 	"math"
 	"sync"
+	"sync/atomic"
 	"time"
 
 	"github.com/pborman/uuid"
@@ -72,6 +73,7 @@ type (
 	}
 
 	matchingEngineImpl struct {
+		status               int32
 		taskManager          persistence.TaskManager
 		historyService       historyservice.HistoryServiceClient
 		matchingClient       matchingservice.MatchingServiceClient
@@ -117,6 +119,7 @@ func NewEngine(taskManager persistence.TaskManager,
 ) Engine {
 
 	return &matchingEngineImpl{
+		status:               common.DaemonStatusInitialized,
 		taskManager:          taskManager,
 		historyService:       historyService,
 		tokenSerializer:      common.NewProtoTaskTokenSerializer(),
@@ -132,11 +135,24 @@ func NewEngine(taskManager persistence.TaskManager,
 }
 
 func (e *matchingEngineImpl) Start() {
-	// As task queues are initialized lazily nothing is done on startup at this point.
+	if !atomic.CompareAndSwapInt32(
+		&e.status,
+		common.DaemonStatusInitialized,
+		common.DaemonStatusStarted,
+	) {
+		return
+	}
 }
 
 func (e *matchingEngineImpl) Stop() {
-	// Executes Stop() on each task queue outside of lock
+	if !atomic.CompareAndSwapInt32(
+		&e.status,
+		common.DaemonStatusStarted,
+		common.DaemonStatusStopped,
+	) {
+		return
+	}
+
 	for _, l := range e.getTaskQueues(math.MaxInt32) {
 		l.Stop()
 	}
@@ -192,11 +208,7 @@ func (e *matchingEngineImpl) getTaskQueueManager(taskQueue *taskQueueID, taskQue
 	}
 	e.taskQueues[*taskQueue] = mgr
 	e.taskQueuesLock.Unlock()
-	err = mgr.Start()
-	if err != nil {
-		e.logger.Info("", tag.LifeCycleStartFailed, tag.WorkflowTaskQueueName(taskQueue.name), tag.WorkflowTaskQueueType(taskQueue.taskType), tag.Error(err))
-		return nil, err
-	}
+	mgr.Start()
 	e.logger.Info("", tag.LifeCycleStarted, tag.WorkflowTaskQueueName(taskQueue.name), tag.WorkflowTaskQueueType(taskQueue.taskType))
 	return mgr, nil
 }
@@ -371,7 +383,7 @@ pollLoop:
 			})
 			if err != nil {
 				// will notify query client that the query task failed
-				e.deliverQueryResult(task.query.taskID, &queryResult{internalError: err}) // nolint:errcheck
+				_ = e.deliverQueryResult(task.query.taskID, &queryResult{internalError: err})
 				return emptyPollWorkflowTaskQueueResponse, nil
 			}
 
@@ -410,7 +422,7 @@ pollLoop:
 	}
 }
 
-// pollForActivityTaskOperation takes one task from the task manager, update workflow execution history, mark task as
+// PollActivityTaskQueue takes one task from the task manager, update workflow execution history, mark task as
 // completed and return it to user. If a task from task manager is already started, return an empty response, without
 // error. Timeouts handled by the timer queue.
 func (e *matchingEngineImpl) PollActivityTaskQueue(
@@ -548,7 +560,7 @@ func (e *matchingEngineImpl) RespondQueryTaskCompleted(
 func (e *matchingEngineImpl) deliverQueryResult(taskID string, queryResult *queryResult) error {
 	queryResultCh, ok := e.lockableQueryTaskMap.get(taskID)
 	if !ok {
-		return serviceerror.NewInternal("query task not found, or already expired")
+		return serviceerror.NewNotFound("query task not found, or already expired")
 	}
 	queryResultCh <- queryResult
 	return nil
