@@ -27,10 +27,12 @@ package temporal
 import (
 	"net/http"
 
+	"go.temporal.io/server/client"
 	"go.temporal.io/server/common/authorization"
 	"go.temporal.io/server/common/config"
 	"go.temporal.io/server/common/dynamicconfig"
 	"go.temporal.io/server/common/log"
+	persistenceclient "go.temporal.io/server/common/persistence/client"
 	"go.temporal.io/server/common/resolver"
 	"go.temporal.io/server/common/rpc/encryption"
 )
@@ -73,6 +75,13 @@ func WithLogger(logger log.Logger) ServerOption {
 	})
 }
 
+// Sets optional logger for all frontend operations
+func WithNamespaceLogger(namespaceLogger log.Logger) ServerOption {
+	return newApplyFuncContainer(func(s *serverOptions) {
+		s.namespaceLogger = namespaceLogger
+	})
+}
+
 // Sets low level authorizer to allow/deny all API calls
 func WithAuthorizer(authorizer authorization.Authorizer) ServerOption {
 	return newApplyFuncContainer(func(s *serverOptions) {
@@ -132,3 +141,19 @@ func WithDynamicConfigClient(c dynamicconfig.Client) ServerOption {
 		s.dynamicConfigClient = c
 	})
 }
+
+// WithCustomDataStoreFactory sets a custom AbstractDataStoreFactory
+// NOTE: this option is experimental and may be changed or removed in future release.
+func WithCustomDataStoreFactory(customFactory persistenceclient.AbstractDataStoreFactory) ServerOption {
+	return newApplyFuncContainer(func(s *serverOptions) {
+		s.customDataStoreFactory = customFactory
+	})
+}
+
+// WithClientFactoryProvider sets a custom ClientFactoryProvider
+// NOTE: this option is experimental and may be changed or removed in future release.
+func WithClientFactoryProvider(clientFactoryProvider client.FactoryProvider) ServerOption {
+	return newApplyFuncContainer(func(s *serverOptions) {
+		s.clientFactoryProvider = clientFactoryProvider
+	})
+}
