@@ -599,17 +599,36 @@ func newAdminClusterCommands() []cli.Command {
 				},
 				cli.StringSliceFlag{
 					Name:  FlagNameWithAlias,
-					Usage: "Search attribute name",
+					Usage: "Search attribute name (multiply values are supported)",
 				},
 				cli.StringSliceFlag{
 					Name:  FlagTypeWithAlias,
-					Usage: fmt.Sprintf("Search attribute type: %v", allowedEnumValues(enumspb.IndexedValueType_name)),
+					Usage: fmt.Sprintf("Search attribute type: %v (multiply values are supported)", allowedEnumValues(enumspb.IndexedValueType_name)),
 				},
 			},
 			Action: func(c *cli.Context) {
 				AdminAddSearchAttributes(c)
 			},
 		},
+		{
+			Name:    "remove-search-attributes",
+			Aliases: []string{"rsa"},
+			Usage:   "Remove custom search attributes metadata only (Elasticsearch schema is not modified)",
+			Flags: []cli.Flag{
+				cli.StringFlag{
+					Name:   FlagIndex,
+					Usage:  "ES index name (optional)",
+					Hidden: true, // don't show it for now
+				},
+				cli.StringSliceFlag{
+					Name:  FlagNameWithAlias,
+					Usage: "Search attribute name",
+				},
+			},
+			Action: func(c *cli.Context) {
+				AdminRemoveSearchAttributes(c)
+			},
+		},
 		{
 			Name:    "get-search-attributes",
 			Aliases: []string{"gsa"},
